/*
 *
 * HGA v1.3.1
 * Program: makepop, which generates input files for the HGA
 * File: makepop.h, defines all types and globals for program and 
 * provides prototypes
 *
 * Copyright (c) 1997, 1995, 1994 Stephen D. Scott and the University of
 * Nebraska-Lincoln.  All rights reserved.
 *
 * Permission to use or modify this software and its documentation only
 * for educational purposes and without fee is hereby granted, provided
 * that the above copyright notice appears in all copies.  The author
 * and university make no representations about the suitability of this
 * software for any purpose.  It is provided "as is" without express or
 * implied warranty.  The author and university shall not be liable in
 * any event for incidental or consequential damages in connection
 * with, or arising out of, the furnishing, performance, or use of
 * these programs.
 *
 */

#include <stdio.h>
#include <string.h>
#include <math.h>
#include <stdlib.h>
#include <stdarg.h>
#include <sys/time.h>

#ifndef MAIN
#define EXT extern
#else
#define EXT
#endif

#undef WRITEOLDSEED    /* used to comment out debugging statements */

#define PI 3.1415927

#define min(a,b) (((a) < (b)) ? (a) : (b))
#define log2(x) (log(x)/log(2.0))

#ifdef SPAMSPAMSPAM
#define MAXRANDOM 0x7FFFFFFFL

/* flip a biased coin */
#define flip(prob) (((double)random()) < ((double)(MAXRANDOM) * (prob)) ? '1' : '0')

/* generate random float number in [0,1] */
#define rrandom()  (((double)random())/((double)MAXRANDOM))

/* generate random int in [min,max] */
#define rnd(min,max) ((min)+((int)((((double)random())/((double)MAXRANDOM)) \
                     *((max)-(min)))))
#else

/* flip a biased coin */
#define flip(prob) (((double)rand()) < ((double)(RAND_MAX) * (prob)) ? '1' : '0')

/* generate random float number in [0,1] */
#define rrandom()  (((double)rand())/((double)RAND_MAX))

/* generate random int in [min,max] */
#define rnd(min,max) ((min)+((unsigned long long)((((double)rand())/((double)RAND_MAX)) \
                     *((max)-(min)))))
#endif

/* macro to flip a bit */
#define not(x) ((x) ? 0 : 1)

EXT int p, n, f, m, g, valw, memsize, oldseed, useseed;
EXT double mutprob, xovprob;
EXT FILE *out;
EXT char *fname;

void CheckArgs(int argc, char *argv[]);
void Usage(char *invalidarg, char *name);
void Error(char *message);
void Init(void);
int ToBinary(unsigned long long val, char *string, int size);
void Output(void);
void initrnd(void);
int Eval(unsigned long long value);
unsigned long long mypow(int, int);
